;; lego-logo.scm --- Create LEGO(R) LOGO
;;
;; Author: KATO Kazuyoshi <kzys@users.sf.net>

(define (floor x) (- x (fmod x 1)))

(define (apply-lego-logo-effect image text-layer tcolor bcolor border1 border2 square-background sbcolor)
  (let* ((text-width (car (gimp-drawable-width text-layer)))
	 (text-height (car (gimp-drawable-height text-layer)))

	 (background-layer
	  (car (gimp-layer-new image text-width text-height RGBA-IMAGE
			       "Background" 100 NORMAL)))
	 (yellow-border-layer
	  (car (gimp-layer-new image text-width text-height RGBA-IMAGE
			       "Yellow Border" 100 NORMAL)))

	 (black-border-layer
	  (car (gimp-layer-new image text-width text-height RGBA-IMAGE
			       "Black Border" 100 NORMAL)))

	 (old-foreground (car (gimp-palette-get-foreground)))
	 (old-background (car (gimp-palette-get-background))))


    ;; resize
    (if (= square-background TRUE)
	(let* ((width (max text-width text-height))
	       (height width))
	  (gimp-image-resize image width height 0 0)

	  (gimp-layer-translate text-layer
				(/ (- width text-width) 2)
				(/ (- height text-height) 2))
	  (mapcar
	   (lambda (layer)
	     (gimp-image-add-layer image layer 1)	;ړ(2.2)
	     (gimp-layer-resize layer width height 0 0))
	   ;̏ɃC[ɒǉ(2.2)
	   (list background-layer yellow-border-layer black-border-layer)
	  ))

	(gimp-image-resize image text-width text-height 0 0))

    ;; white text
    (gimp-layer-set-preserve-trans text-layer TRUE)
    (gimp-palette-set-foreground tcolor)
    (gimp-edit-fill text-layer FG-IMAGE-FILL)

    ;; add border layers
    (gimp-selection-none image)
    (mapcar
     (lambda (layer)
;       (gimp-image-add-layer image layer 2)	;ړ(2.2)
       (gimp-edit-clear layer))
     (list black-border-layer yellow-border-layer))

    ;; draw border
    (gimp-selection-layer-alpha text-layer)
    (mapcar
     (lambda (layer color)
       (gimp-selection-grow image (floor (* text-height 0.04)))
       (gimp-palette-set-foreground color)
       (gimp-edit-fill layer FG-IMAGE-FILL))
     (list black-border-layer yellow-border-layer)
     (list border1 border2))

    ;; red background
;    (gimp-image-add-layer image background-layer 3)	;(2.2)
    (gimp-selection-all image)
    (gimp-palette-set-foreground bcolor)
    (gimp-edit-fill background-layer FG-IMAGE-FILL)

    (if (= square-background TRUE)
        (begin
          (gimp-selection-all image)
          (gimp-selection-shrink image (floor (* text-height 0.05)))
          (gimp-selection-invert image)
          (gimp-palette-set-foreground sbcolor)
          (gimp-edit-fill background-layer FG-IMAGE-FILL)))
  
    (gimp-selection-none image)
    (gimp-palette-set-background old-background)
    (gimp-palette-set-foreground old-foreground)))

(define (script-fu-lego-logo text size font tcolor bcolor border1 border2 square-background sbcolor)
  (let* ((image (car (gimp-image-new 256 256 RGB)))
	 (text-layer
	  (car
	   (gimp-text-fontname image -1 0 0 text
			       (floor (* size 0.4)) TRUE size PIXELS font))))
    (gimp-image-undo-disable image)

    (gimp-layer-set-name text-layer text)
    (apply-lego-logo-effect image text-layer tcolor bcolor border1 border2 square-background sbcolor)

    (gimp-image-undo-enable image)
    (gimp-display-new image)))

(script-fu-register "script-fu-lego-logo"
		    _"<Image>/File/Create/Logos/Lego Logo..."
		    "Create LEGO(R) logo"
		    "KATO Kazuyoshi"
		    "KATO Kazuyoshi"
		    "2004"
		    ""
		    SF-STRING     _"Text" "LEGO"
		    SF-ADJUSTMENT _"Font Size (pixels)" '(80 2 1000 1 10 0 1)
		    SF-FONT       _"Font" "Comic Sans MS"
                    SF-COLOR       "Text Color"      '(255 255 255)
                    SF-COLOR       "Background"      '(255 0 0)
                    SF-COLOR       "Border1"         '(0 0 0)
                    SF-COLOR       "Border2"         '(255 240 0)
		    SF-TOGGLE     _"Square Background" TRUE
                    SF-COLOR       "sbcolor"         '(0 0 0))
