; Multi-Text Effect V1.0 (11-2007) (c) Copyright
; Created by Scott Mosteller  
; Comments directed to http://www.gimpscripts.com
;
; Create text with a choice of multiple effects
; 
; Simply enter the text and select the effect type. The scrip will generate 4 frames with the selected effect
; You can control various aspects of each effect by adjusting the effect tweak parameter and the scatter
; parameter.
; 
; Simply save the generated text as an animated .GIF and you're done.
;
; Selecting Raw Layer will display all raw layers only
; Clicking off animation will display a single frame, flattend to background.
;
; *NOTE: Scatter-HSV is handled differently 2.4 that it was in Gimp 2.2. 
;        When using Gimp 2.2, set scatter lower (40-60)
;        When using Gimp 2.4, set scatter higher (140-160)
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
; Compatible with all Gimp Versions 2.x - Enjoy!
;
; ------------
;| Change Log |
; ------------ 
; V1.0 - Initial Release
;
;
; User Options Popup
;

;Tosilscript-fugp܂B
;http://www.geocities.jp/gimproject/scripts/layers-view-delete.html

(define (script-fu-layers-view-none image drawable)
 (let* ((layers (gimp-image-get-layers image))
	    (number-layers (car layers))
	    (layer-array (cadr layers))
	    (layer-count 0)
	    (layer))
    (set! layer-count 0)
    (while (< layer-count number-layers)
           (set! layer (aref layer-array layer-count))
           (gimp-layer-set-visible layer FALSE)
           (set! layer-count (+ layer-count 1)))
))
(define (script-fu-layers-view-current-only image drawable)
    (script-fu-layers-view-none image drawable)
    (if (not (= 0 (car (gimp-drawable-is-layer drawable))))
        (gimp-layer-set-visible drawable TRUE))
)
(define (script-fu-layers-delete-hidden image drawable)
 (let* ((layers (gimp-image-get-layers image))
	    (number-layers (car layers))
	    (layer-array (cadr layers))
	    (layer-count 0)
	    (layer))
    (gimp-image-undo-group-start image)
    (set! layer-count 0)
    (while (< layer-count number-layers)
           (set! layer (aref layer-array layer-count))
           (if (= (car (gimp-layer-get-visible layer)) FALSE)
               (gimp-image-remove-layer image layer))
           (set! layer-count (+ layer-count 1)))
    (gimp-image-undo-group-end image)
    (gimp-displays-flush)))

(script-fu-register "script-fu-multi-effect-text"
		    _"<Image>/File/Create/Logos/MultiEffect Text..."
		    "Creates a Multi-Effect Text with a drop shadow"
		    "Scott Mosteller"
		    "Scott Mosteller"
		    "2007"
		    ""
		    SF-STRING     _"Text"               "MultiEffect Text"
		    SF-ADJUSTMENT _"Font size (pixels)" '(150 2 1000 1 10 0 1)
		    SF-FONT       _"Font"               "Arial Bold"
		    SF-COLOR      _"Text color"         '(195 8 8)
		    SF-COLOR      _"Stroke color color" '(0 0 0)
                    SF-ADJUSTMENT _"Stroke width"       '(2 1 50 1 1 0 1)
		    SF-COLOR      _"Shadow color"       '(0 0 0)
		    SF-COLOR      _"Background color"   '(255 255 255)
                    SF-ADJUSTMENT _"Shadow Offset X"    '(4 0 100 1 1 0 1)
                    SF-ADJUSTMENT _"Shadow Offset Y"    '(4 0 100 1 1 0 1)
                    SF-ADJUSTMENT _"Shadow blur"	'(5 0 100 1 1 0 1)
                    SF-ADJUSTMENT _"Shadow Opacity"     '(60 1 100 1 1 0 1)
                    SF-OPTION     _"Effect Type"       '("Cubism" "Oilify" "Plasma" "Diffraction" "No Effect")
                    SF-ADJUSTMENT _"Effect Tweak"       '(4.5 1 20 .1 .1 1 0)
                    SF-ADJUSTMENT _"Scatter Amount"     '(150 1 180 1 1 0 0)
                    SF-TOGGLE     _"Animate?"           TRUE
                    SF-TOGGLE     _"Raw Layers Only?(not-semiflatten)"   FALSE
                    SF-TOGGLE     _"Background Transparent?"   FALSE
)
;

;
; Define Main Multi-effect Text Function
;
(define (script-fu-multi-effect-text text
			       size
			       font
			       text-color
			       stroke-color
                               stroke-width
                               shadow-color
			       bg-color
                               shx
                               shy
			       sblur
                               sho
                               etype
                               tweak
                               scatter
                               lani
                               raw
			       trans?)
  (let* ((img (car (gimp-image-new 256 256 RGB)))
    (tmp (car (gimp-palette-set-foreground stroke-color)))
    (tmpfg (car (gimp-palette-get-foreground img)))
    (tmpbg (car (gimp-palette-get-background img)))
    (text-layer (car (gimp-text-fontname img -1 0 0 text (+ (/ size 12) 4) TRUE size PIXELS font)))
;    (text-layer2 (car (gimp-text-fontname img -1 0 0 text 10 TRUE size PIXELS font)))
;    (text-layer3 (car (gimp-text-fontname img -1 0 0 text 10 TRUE size PIXELS font)))
;    (text-layer4 (car (gimp-text-fontname img -1 0 0 text 10 TRUE size PIXELS font)))
    )
;
; Multi-effect  Text Main Procedure Body
;
(gimp-drawable-set-name text-layer text)

(script-fu-multi-effect-textcore img
			       text-layer
			       size
			       text-color
			       stroke-color
                               stroke-width
                               shadow-color
			       bg-color
                               shx
                               shy
			       sblur
                               sho
                               etype
                               tweak
                               scatter
                               lani
                               raw
			       trans?)

   (gimp-palette-set-foreground tmpfg)
   (gimp-palette-set-background tmpbg)
    (gimp-image-undo-enable img)
    (gimp-display-new img)))
;
; End Multi-effect  Text Main Procedure
;


(define (script-fu-multi-effect-textimg img
			       text-layer
			       size
			       text-color
			       stroke-color
                               stroke-width
                               shadow-color
			       bg-color
                               shx
                               shy
			       sblur
                               sho
                               etype
                               tweak
                               scatter
                               lani
                               raw
			       trans?)
  (let* (
    (tmpfg (car (gimp-palette-get-foreground img)))
    (tmpbg (car (gimp-palette-get-background img)))
    (tmp (car (gimp-palette-set-foreground stroke-color)))
    )

(script-fu-multi-effect-textcore img
			       text-layer
			       size
			       text-color
			       stroke-color
                               stroke-width
                               shadow-color
			       bg-color
                               shx
                               shy
			       sblur
                               sho
                               etype
                               tweak
                               scatter
                               lani
                               raw
			       trans?)

)
)

;摜H{
(define (script-fu-multi-effect-textimgh img
			       text-layer
			       size
			       text-color
			       stroke-color
                               stroke-width
                               shadow-color
			       bg-color
                               shx
                               shy
			       sblur
                               sho
                               etype
                               tweak
                               scatter
                               lani
                               raw
			       trans?)
  (let* (
	(img2 (car (gimp-channel-ops-duplicate img)))
	(drawable2 (car (gimp-image-get-active-layer img2)))
	(tmpfg (car (gimp-palette-get-foreground img)))
	(tmpbg (car (gimp-palette-get-background img)))
	)
	(gimp-image-undo-disable img2)
	(gimp-selection-layer-alpha drawable2)		;add
	(gimp-palette-set-foreground '(0 0 0))		;add
	(gimp-edit-fill drawable2 FOREGROUND-FILL)	;add
	(gimp-selection-none img2)			;add

	(script-fu-layers-view-current-only img2 drawable2)	;Tosilscript-fu
	(script-fu-layers-delete-hidden img2 drawable2)		;Tosilscript-fu

	(script-fu-multi-effect-textimg img2
			       drawable2
			       size
			       text-color
			       stroke-color
                               stroke-width
                               shadow-color
			       bg-color
                               shx
                               shy
			       sblur
                               sho
                               etype
                               tweak
                               scatter
                               lani
                               raw
			       trans?)

	(gimp-palette-set-foreground tmpfg)
	(gimp-palette-set-background tmpbg)
	(gimp-image-undo-enable img2)
	(gimp-display-new img2)
  )
)

(script-fu-register "script-fu-multi-effect-textimgh"
		    _"<Image>/Filters/Animation/MultiEffect..."
		    "Creates a Multi-Effect Text with a drop shadow"
		    "Scott Mosteller"
		    "Scott Mosteller"
		    "2007"
		    "RGBA"
		SF-IMAGE       	"Image" 0
		SF-DRAWABLE    	"Drawable" 0
		    SF-ADJUSTMENT _"size (pixels)" '(150 2 1000 1 10 0 1)
		    SF-COLOR      _"Color"		'(195 8 8)
		    SF-COLOR      _"Stroke color color" '(0 0 0)
                    SF-ADJUSTMENT _"Stroke width"       '(2 1 50 1 1 0 1)
		    SF-COLOR      _"Shadow color"       '(0 0 0)
		    SF-COLOR      _"Background color"   '(255 255 255)
                    SF-ADJUSTMENT _"Shadow Offset X"    '(4 0 100 1 1 0 1)
                    SF-ADJUSTMENT _"Shadow Offset Y"    '(4 0 100 1 1 0 1)
                    SF-ADJUSTMENT _"Shadow blur"	'(5 0 100 1 1 0 1)
                    SF-ADJUSTMENT _"Shadow Opacity"     '(60 1 100 1 1 0 1)
                    SF-OPTION     _"Effect Type"       '("Cubism" "Oilify" "Plasma" "Diffraction" "No Effect")
                    SF-ADJUSTMENT _"Effect Tweak"       '(4.5 1 20 .1 .1 1 0)
                    SF-ADJUSTMENT _"Scatter Amount"     '(150 1 180 1 1 0 0)
                    SF-TOGGLE     _"Animate?"           TRUE
                    SF-TOGGLE     _"Raw Layers Only?(not-semiflatten)"   FALSE
                    SF-TOGGLE     _"Background Transparent?"   FALSE
)


;
; Define Multi-effect Text Apply Routine
;
;
(define (apply-multi-effect-text img
				  logo-layer
				  text-color
                                  stroke-color
                                  stroke-width
                                  shadow-color
				  bg-color
                                  shx
                                  shy
				  sblur
                                  sho
                                  etype
                                  tweak
                                  scatter
                                  lani
                                  raw
                                  tmpfg
                                  tmpbg
                                  diffinc)
  (let* ((width (car (gimp-drawable-width logo-layer)))
	 (height (car (gimp-drawable-height logo-layer)))
	 (shadow-layer (car (gimp-layer-new img width height RGBA-IMAGE "Shadow" sho NORMAL-MODE)))
         (df1)
         (df2)
         (df3)
         (df4)
         (df5)
         (df6)
	 (lwidth)
	 (lheight)
     )
;
; Subroutine - Multi-effect  Text Apply Body
;

   (gimp-selection-none img)
   (script-fu-util-image-resize-from-layer img logo-layer)

;
; Set color for active gradient & call effects procedure
;
   (gimp-palette-set-foreground tmpfg)
   (gimp-palette-set-background tmpbg)


;
; Fill text layer with foreground color
;
  (gimp-by-color-select logo-layer stroke-color 15 2 TRUE FALSE 10 FALSE)
  (gimp-selection-shrink img stroke-width)
  (gimp-palette-set-foreground text-color)
  (gimp-edit-bucket-fill logo-layer 0 0 100 0 0 0 0)

;  (script-fu-add-bevel img logo-layer 20 0 0)
;
;
; Effect #1 - Cubism
;
(if (= etype 0)
   (begin
   (plug-in-scatter-hsv 1 img logo-layer 1 scatter 10 10)
   (set! tweak (+ tweak 7))
   (plug-in-cubism 1 img logo-layer tweak 2 0)
   ))
;
; Effect #2 - Oilify
;
(if (= etype 1)
   (begin
   (plug-in-scatter-hsv 1 img logo-layer 1 scatter 10 10)
   (plug-in-oilify 1 img logo-layer tweak 0)
   ))
;
; Effect #2 - Plasma
;
(if (= etype 2)
   (begin
   (plug-in-scatter-hsv 1 img logo-layer 1 scatter 10 10)
   (plug-in-plasma 1 img logo-layer (rand 999999999) tweak)
   ))
;
; Effect #3 - Diffraction
;
;

(if (= etype 3)
   (begin
   (set! df1 (+ .821 diffinc)) ;Calculate parameters using tweak value
   (set! df2 (+ 1.221 diffinc))
   (set! df3 (+ 1.123 diffinc))
   (set! df4 (+ .821 (/ diffinc 2)))
   (set! df5 (+ .821 (/ diffinc 2)))
   (set! df6 (+ .974 (/ diffinc 2)))
   (plug-in-diffraction 1 img logo-layer df1 df2 df3 df4 df5 df6 0.610 0.677 0.636 0.066 37.126 -0.473)
   ))
;
; Create shadow layer
;
   (gimp-selection-none img)
   (gimp-image-add-layer img shadow-layer 1)
   (gimp-layer-resize-to-image-size shadow-layer)
   (gimp-edit-clear shadow-layer)
   (gimp-selection-layer-alpha logo-layer)
   (if (>= sblur 1) (gimp-selection-feather img sblur))
   (gimp-palette-set-background shadow-color)
   (gimp-edit-fill shadow-layer BACKGROUND-FILL)
   (gimp-layer-translate shadow-layer shx shy)

   (gimp-image-lower-layer img shadow-layer) ; Drop shadow to bottom
   (gimp-image-lower-layer img shadow-layer)
   (gimp-image-lower-layer img shadow-layer)
;;;   (gimp-image-resize-to-layers img)				;none 2.0
))
;
; End Subroutine Apply     


(define (script-fu-multi-effect-textcore img
			       text-layer
			       size
			       text-color
			       stroke-color
                               stroke-width
                               shadow-color
			       bg-color
                               shx
                               shy
			       sblur
                               sho
                               etype
                               tweak
                               scatter
                               lani
                               raw
			       trans?)
(let* (
    (text-layer2 (car (gimp-layer-copy text-layer 1)))	;add
    (text-layer3 (car (gimp-layer-copy text-layer 1)))	;add
    (text-layer4 (car (gimp-layer-copy text-layer 1)))	;add
    (width (car (gimp-drawable-width text-layer)))
    (height (car (gimp-drawable-height text-layer)))
    (bg-layer (car (gimp-layer-new img (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) RGB-IMAGE "Background" 100 NORMAL)))
    (bg-layer2 (car (gimp-layer-new img (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) RGB-IMAGE "Background" 100 NORMAL)))
    (bg-layer3 (car (gimp-layer-new img (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) RGB-IMAGE "Background" 100 NORMAL)))
    (bg-layer4 (car (gimp-layer-new img (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) RGB-IMAGE "Background" 100 NORMAL)))
    (lay1)
    (lay2)
    (lay3)
    (lay4)
    (diffinc)
    (dummy1 (car (gimp-layer-new img width height RGBA-IMAGE "dummy" 100 NORMAL)))
    (dummy2 (car (gimp-layer-new img width height RGBA-IMAGE "dummy" 100 NORMAL)))
    (dummy3 (car (gimp-layer-new img width height RGBA-IMAGE "dummy" 100 NORMAL)))
    (dummy4 (car (gimp-layer-new img width height RGBA-IMAGE "dummy" 100 NORMAL)))
    (shou 0)
    (tmpfg (car (gimp-palette-get-foreground img)))
    (tmpbg (car (gimp-palette-get-background img)))
    )

;
; Turn off text layers 2,3,4
;
    (gimp-image-add-layer img text-layer2 0)		;add
    (gimp-image-add-layer img text-layer3 0)		;add
    (gimp-image-add-layer img text-layer4 0)		;add
    (gimp-drawable-set-visible text-layer2 0)
    (gimp-drawable-set-visible text-layer3 0)
    (gimp-drawable-set-visible text-layer4 0)
;
; Create animation layer 1
;
(set! diffinc (- tweak 1))
(gimp-image-undo-disable img)
(gimp-image-resize img (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) (/ (* sblur 1.1) 2) (/ (* sblur 1.1) 2))
(gimp-layer-resize text-layer (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) (/ (* sblur 1.1) 2) (/ (* sblur 1.1) 2))
(apply-multi-effect-text img text-layer text-color stroke-color stroke-width shadow-color bg-color shx shy sblur sho etype tweak scatter lani raw tmpfg tmpbg diffinc)
(if (= FALSE trans?)
(begin
   (gimp-image-add-layer img bg-layer 6)		;add
   (gimp-selection-none img)				;add
   (gimp-palette-set-background bg-color)		;add
   (gimp-edit-fill bg-layer BACKGROUND-FILL)		;add
   (set! lay1 (car (gimp-image-merge-visible-layers img 0)))	;add
   (if (= raw TRUE) (gimp-drawable-set-visible lay1 0))		;add
))
(if (= raw FALSE)
  (begin
   (gimp-image-add-layer img dummy1 0)			;add
    (gimp-drawable-fill dummy1 TRANS-IMAGE-FILL)	;add    noise remove
   (set! lay1 (car (gimp-image-merge-visible-layers img 0)))
   (gimp-drawable-set-name lay1 "frame1")
   (gimp-selection-none img)
   (gimp-palette-set-background bg-color)
   (gimp-image-set-active-layer img lay1)
   (plug-in-semiflatten 1 img lay1)
   (gimp-drawable-set-visible lay1 0)
))
(if (and (= TRUE trans?) (= raw TRUE))
(begin
	(set! lay1 (car (gimp-image-merge-visible-layers img 0)))
	(gimp-drawable-set-visible lay1 0)
))
;
; Skip if not animated
;
(if (= lani TRUE)
  (begin
;
; Create animation layer 2
;
(set! diffinc (- (+ tweak 0.35) 1))
(gimp-drawable-set-visible text-layer2 1)
(gimp-layer-resize text-layer2 (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) (/ (* sblur 1.1) 2) (/ (* sblur 1.1) 2))
(apply-multi-effect-text img text-layer2 text-color stroke-color stroke-width shadow-color bg-color shx shy sblur sho etype tweak scatter lani raw tmpfg tmpbg diffinc)
(if (= FALSE trans?)
(begin
   (gimp-image-add-layer img bg-layer2 5)		;add
   (gimp-selection-none img)				;add
   (gimp-palette-set-background bg-color)		;add
   (gimp-edit-fill bg-layer2 BACKGROUND-FILL)		;add
   (set! lay2 (car (gimp-image-merge-visible-layers img 0)))	;add
   (if (= raw TRUE) (gimp-drawable-set-visible lay2 0))		;add
))
(if (= raw FALSE)
  (begin
   (gimp-image-add-layer img dummy2 0)			;add
    (gimp-drawable-fill dummy2 TRANS-IMAGE-FILL)	;add    noise remove
   (set! lay2 (car (gimp-image-merge-visible-layers img 0)))
   (gimp-drawable-set-name lay2 "frame2")
   (gimp-selection-none img)
   (gimp-palette-set-background bg-color)
   (gimp-image-set-active-layer img lay2)
   (plug-in-semiflatten 1 img lay2)
   (gimp-drawable-set-visible lay2 0)
))
(if (and (= TRUE trans?) (= raw TRUE))
(begin
	(set! lay2 (car (gimp-image-merge-visible-layers img 0)))
	(gimp-drawable-set-visible lay2 0)
))
;
; Create animation layer 3
;
(set! diffinc (- (+ tweak 0.7) 1))
(gimp-drawable-set-visible text-layer3 1)
(gimp-layer-resize text-layer3 (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) (/ (* sblur 1.1) 2) (/ (* sblur 1.1) 2))
(apply-multi-effect-text img text-layer3 text-color stroke-color stroke-width shadow-color bg-color shx shy sblur sho etype tweak scatter lani raw tmpfg tmpbg diffinc)
(if (= FALSE trans?)
(begin
   (gimp-image-add-layer img bg-layer3 5)		;add
   (gimp-selection-none img)				;add
   (gimp-palette-set-background bg-color)		;add
   (gimp-edit-fill bg-layer3 BACKGROUND-FILL)		;add
   (set! lay3 (car (gimp-image-merge-visible-layers img 0)))	;add
   (if (= raw TRUE) (gimp-drawable-set-visible lay3 0))		;add
))
(if (= raw FALSE)
  (begin
   (gimp-image-add-layer img dummy3 0)			;add
    (gimp-drawable-fill dummy3 TRANS-IMAGE-FILL)	;add    noise remove
   (set! lay3 (car (gimp-image-merge-visible-layers img 0)))
   (gimp-drawable-set-name lay3 "frame3")
   (gimp-selection-none img)
   (gimp-palette-set-background bg-color)
   (gimp-image-set-active-layer img lay3)
;   (plug-in-semiflatten 1 img lay3)			;none
   (gimp-drawable-set-visible lay3 0)
))
(if (and (= TRUE trans?) (= raw TRUE))
(begin
	(set! lay3 (car (gimp-image-merge-visible-layers img 0)))
	(gimp-drawable-set-visible lay3 0)
))
;
; Create animation layer 4
;
(set! diffinc (- (+ tweak 1.05) 1))
(gimp-drawable-set-visible text-layer4 1)
(gimp-layer-resize text-layer4 (+ (+ width shx) (* sblur 1.1)) (+ (+ height shy) (* sblur 1.1)) (/ (* sblur 1.1) 2) (/ (* sblur 1.1) 2))
(apply-multi-effect-text img text-layer4 text-color stroke-color stroke-width shadow-color bg-color shx shy sblur sho etype tweak scatter lani raw tmpfg tmpbg diffinc)
(if (= FALSE trans?)
(begin
   (gimp-image-add-layer img bg-layer4 5)		;add
   (gimp-selection-none img)				;add
   (gimp-palette-set-background bg-color)		;add
   (gimp-edit-fill bg-layer4 BACKGROUND-FILL)		;add
   (set! lay4 (car (gimp-image-merge-visible-layers img 0)))	;add
   (if (= raw TRUE) (gimp-drawable-set-visible lay4 0))		;add
))
(if (= raw FALSE)
  (begin
   (gimp-image-add-layer img dummy4 0)			;add
    (gimp-drawable-fill dummy4 TRANS-IMAGE-FILL)	;add    noise remove
    (set! lay4 (car (gimp-image-merge-visible-layers img 0)))
    (gimp-drawable-set-name lay4 "frame4")
    (gimp-selection-none img)
    (gimp-palette-set-background bg-color)
    (gimp-image-set-active-layer img lay4)
   (plug-in-semiflatten 1 img lay4)
    (gimp-drawable-set-visible lay4 0)
))
(if (and (= TRUE trans?) (= raw TRUE))
(begin
	(set! lay4 (car (gimp-image-merge-visible-layers img 0)))
	(gimp-drawable-set-visible lay4 0)
))
))

;
; Raw and animation cleanup
;
;(if (= raw FALSE)
;  (begin
    (gimp-drawable-set-visible lay1 1)
;    (plug-in-autocrop 1 img lay1)			;none
     (if (= lani TRUE)
       (begin
         (gimp-drawable-set-visible lay2 1)
         (gimp-drawable-set-visible lay3 1)
         (gimp-drawable-set-visible lay4 1)
        )
        (begin
         (gimp-selection-none img)
         (gimp-image-raise-layer-to-top img lay1)
	 (gimp-image-remove-layer img text-layer2)	;add
	 (gimp-image-remove-layer img text-layer3)	;add
	 (gimp-image-remove-layer img text-layer4)	;add
;         (set! lay1 (car (gimp-image-flatten img)))		;none
    ))

    (set! width (car (gimp-image-width img)))			;add
    (set! height (car (gimp-image-height img)))			;add
    (if (and (>= shx shy) (>= sblur shy))			;add
	(set! shou shy)						;add
    )								;add
    (if (and (>= shy shx) (>= sblur shx))			;add
	(set! shou shx)						;add
    )								;add
    (if (and (>= shy sblur) (>= shx sblur))			;add
	(set! shou sblur)					;add
    )								;add
    (gimp-image-resize img (- width (/ shou 2))  (- height (/ shou 2)) (- 0 (/ shou 2)) (- 0 (/ shou 2)))
    (gimp-layer-resize lay1 (- width (/ shou 2))  (- height (/ shou 2)) (- 0 (/ shou 2)) (- 0 (/ shou 2)))
(if (= lani TRUE)
  (begin
    (gimp-layer-resize lay2 (- width (/ shou 2))  (- height (/ shou 2)) (- 0 (/ shou 2)) (- 0 (/ shou 2)))
    (gimp-layer-resize lay3 (- width (/ shou 2))  (- height (/ shou 2)) (- 0 (/ shou 2)) (- 0 (/ shou 2)))
    (gimp-layer-resize lay4 (- width (/ shou 2))  (- height (/ shou 2)) (- 0 (/ shou 2)) (- 0 (/ shou 2)))
))

; ))
;
; Clear selection, enable undo and display image
;
    (gimp-selection-none img)
)
)
